public inbox for kernel-tls-handshake@lists.linux.dev
 help / color / mirror / Atom feed
From: Hannes Reinecke <hare@suse.de>
To: Chuck Lever <chuck.lever@oracle.com>
Cc: kernel-tls-handshake@lists.linux.dev, Hannes Reinecke <hare@suse.de>
Subject: [PATCH 3/4] net/tls_handshake: split tls_server_hello()
Date: Fri, 17 Feb 2023 12:31:44 +0100	[thread overview]
Message-ID: <20230217113145.18916-4-hare@suse.de> (raw)
In-Reply-To: <20230217113145.18916-1-hare@suse.de>

Split tls_server_hello() into 'tls_server_hello_x509()' and 'tls_server_hello_psk()'
to handle X.509 and PSK serverhandshakes.

Signed-off-by: Hannes Reinecke <hare@suse.de>
---
 include/net/tls.h       |  8 ++++--
 net/tls/tls_handshake.c | 55 ++++++++++++++++++++++++++++++++++++-----
 2 files changed, 55 insertions(+), 8 deletions(-)

diff --git a/include/net/tls.h b/include/net/tls.h
index f4baf3b4b179..be65cbf72f8a 100644
--- a/include/net/tls.h
+++ b/include/net/tls.h
@@ -540,7 +540,11 @@ int tls_client_hello_x509(struct socket *sock, tls_done_func_t done,
 int tls_client_hello_psk(struct socket *sock, tls_done_func_t done,
 			 void *data, const char *priorities,
 			 key_serial_t peerid, unsigned int timeout);
-int tls_server_hello(struct socket *sock, tls_done_func_t done,
-		     void *data, const char *priorities, unsigned int timeout);
+int tls_server_hello_x509(struct socket *sock, tls_done_func_t done,
+			  void *data, const char *priorities,
+			  unsigned int timeout);
+int tls_server_hello_psk(struct socket *sock, tls_done_func_t done,
+			 void *data, const char *priorities,
+			 unsigned int timeout);
 
 #endif /* _TLS_OFFLOAD_H */
diff --git a/net/tls/tls_handshake.c b/net/tls/tls_handshake.c
index 8171b3c8f3a5..93bb3deaf2fb 100644
--- a/net/tls/tls_handshake.c
+++ b/net/tls/tls_handshake.c
@@ -141,7 +141,7 @@ static int tls_handshake_put_accept_resp(struct sk_buff *msg,
 		goto out;
 
 	ret = nla_put_u32(msg, HANDSHAKE_GENL_ATTR_TLS_TYPE,
-			  HANDSHAKE_GENL_TLS_TYPE_CLIENTHELLO);
+			  treq->th_type);
 	if (ret < 0)
 		goto out;
 	ret = nla_put_u32(msg, HANDSHAKE_GENL_ATTR_TLS_AUTH,
@@ -377,7 +377,7 @@ int tls_client_hello_psk(struct socket *sock, tls_done_func_t done,
 EXPORT_SYMBOL(tls_client_hello_psk);
 
 /**
  * tls_server_hello - request a server TLS handshake on a socket
  * @sock: connected socket on which to perform the handshake
  * @done: function to call when the handshake has completed
  * @data: token to pass back to @done
@@ -389,8 +389,9 @@ EXPORT_SYMBOL(tls_client_hello_psk);
  *   %-ENOENT: No user agent is available
  *   %-ENOMEM: Memory allocation failed
  */
-int tls_server_hello(struct socket *sock, tls_done_func_t done,
-		     void *data, const char *priorities, unsigned int timeout)
+int tls_server_hello_x509(struct socket *sock, tls_done_func_t done,
+			  void *data, const char *priorities,
+			  unsigned int timeout)
 {
 	struct tls_handshake_req *treq;
 	struct handshake_req *req;
@@ -409,10 +410,52 @@ int tls_server_hello(struct socket *sock, tls_done_func_t done,
 
 	treq = tls_handshake_req_init(req, done, data, tp);
 	treq->th_type = HANDSHAKE_GENL_TLS_TYPE_SERVERHELLO;
-	treq->th_auth_type = HANDSHAKE_GENL_TLS_AUTH_UNSPEC;
+	treq->th_auth_type = HANDSHAKE_GENL_TLS_AUTH_X509;
+	if (timeout)
+		treq->th_timeout = timeout;
+
+	return handshake_req_submit(req, flags);
+}
+EXPORT_SYMBOL(tls_server_hello_x509);
+
+/**
+ * tls_server_hello_psk - request a PSK server TLS handshake on a socket
+ * @sock: connected socket on which to perform the handshake
+ * @done: function to call when the handshake has completed
+ * @data: token to pass back to @done
+ * @priorities: GnuTLS TLS priorities string
+ * @timeout: TLS handshake timeout (in seconds)
+ *
+ * Return values:
+ *   %0: Handshake request enqueue; ->done will be called when complete
+ *   %-ENOENT: No user agent is available
+ *   %-ENOMEM: Memory allocation failed
+ */
+int tls_server_hello_psk(struct socket *sock, tls_done_func_t done,
+			 void *data, const char *priorities,
+			 unsigned int timeout)
+{
+	struct tls_handshake_req *treq;
+	struct handshake_req *req;
+	gfp_t flags = GFP_KERNEL;
+	const char *tp;
+
+	tp = tls_handshake_dup_priorities(priorities, flags);
+	if (!tp)
+		return -ENOMEM;
+
+	req = handshake_req_alloc(sock, &tls_handshake_proto, flags);
+	if (!req) {
+		kfree(tp);
+		return -ENOMEM;
+	}
+
+	treq = tls_handshake_req_init(req, done, data, tp);
+	treq->th_type = HANDSHAKE_GENL_TLS_TYPE_SERVERHELLO;
+	treq->th_auth_type = HANDSHAKE_GENL_TLS_AUTH_PSK;
 	if (timeout)
 		treq->th_timeout = timeout;
 
 	return handshake_req_submit(req, flags);
 }
-EXPORT_SYMBOL(tls_server_hello);
+EXPORT_SYMBOL(tls_server_hello_psk);
@@ -377,7 +377,7 @@ int tls_client_hello_psk(struct socket *sock, tls_done_func_t done,
 
 /**
- * tls_server_hello - request a server TLS handshake on a socket
+ * tls_server_hello_x509 - request a X.509 server TLS handshake on a socket
  * @sock: connected socket on which to perform the handshake
  * @done: function to call when the handshake has completed
-- 
2.35.3


  parent reply	other threads:[~2023-02-17 11:31 UTC|newest]

Thread overview: 16+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2023-02-17 11:31 [PATCH 0/4] tls-handshake: server-side support Hannes Reinecke
2023-02-17 11:31 ` [PATCH 1/4] tls-handshake: add 'timeout' netlink attribute Hannes Reinecke
2023-02-17 11:58   ` Chuck Lever III
2023-02-17 12:17     ` Hannes Reinecke
2023-02-17 12:26       ` Chuck Lever III
2023-02-17 12:36         ` Hannes Reinecke
2023-02-17 12:38           ` Chuck Lever III
2023-02-17 11:31 ` [PATCH 2/4] tls-handshake: add 'keyring' " Hannes Reinecke
2023-02-17 12:00   ` Chuck Lever III
2023-02-17 12:24     ` Hannes Reinecke
2023-02-17 12:32       ` Chuck Lever III
2023-02-17 12:48         ` Hannes Reinecke
2023-02-17 12:56           ` Chuck Lever III
2023-02-17 11:31 ` Hannes Reinecke [this message]
2023-02-17 11:31 ` [PATCH 4/4] tls_handshake: add 'keyring' argument to server hello Hannes Reinecke
2023-02-17 11:56 ` [PATCH 0/4] tls-handshake: server-side support Chuck Lever III

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20230217113145.18916-4-hare@suse.de \
    --to=hare@suse.de \
    --cc=chuck.lever@oracle.com \
    --cc=kernel-tls-handshake@lists.linux.dev \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox